@@ -1,21 +1,22 @@
 use std::io::{self, Read};
 
 use ll;
+use ::parse_code;
 
 struct DecoderContext {
-    c: ll::ZBUFFDecompressionContext,
+    s: *mut ll::ZSTD_DStream,
 }
 
 impl Default for DecoderContext {
     fn default() -> Self {
-        DecoderContext { c: unsafe { ll::ZBUFF_createDCtx() } }
+        DecoderContext { s: unsafe { ll::ZSTD_createDStream() } }
     }
 }
 
 impl Drop for DecoderContext {
     fn drop(&mut self) {
-        let code = unsafe { ll::ZBUFF_freeDCtx(self.c) };
-        ll::parse_code(code).unwrap();
+        let code = unsafe { ll::ZSTD_freeDStream(self.s) };
+        parse_code(code).unwrap();
     }
 }
 
@@ -40,7 +41,7 @@ impl<R: Read> Decoder<R> {
     pub fn new(reader: R) -> io::Result<Self> {
         let context = DecoderContext::default();
 
-        try!(ll::parse_code(unsafe { ll::ZBUFF_decompressInit(context.c) }));
+        try!(parse_code(unsafe { ll::ZSTD_initDStream(context.s) }));
 
         Decoder::with_context(reader, context)
     }
@@ -51,17 +52,17 @@ impl<R: Read> Decoder<R> {
     pub fn with_dictionary(reader: R, dictionary: &[u8]) -> io::Result<Self> {
         let context = DecoderContext::default();
 
-        try!(ll::parse_code(unsafe {
-            ll::ZBUFF_decompressInitDictionary(context.c,
-                                               dictionary.as_ptr(),
-                                               dictionary.len())
+        try!(parse_code(unsafe {
+            ll::ZSTD_initDStream_usingDict(context.s,
+                                           dictionary.as_ptr(),
+                                           dictionary.len())
         }));
 
         Decoder::with_context(reader, context)
     }
 
     fn with_context(reader: R, context: DecoderContext) -> io::Result<Self> {
-        let buffer_size = unsafe { ll::ZBUFF_recommendedDInSize() };
+        let buffer_size = unsafe { ll::ZSTD_DStreamInSize() };
 
         Ok(Decoder {
             reader: reader,
@@ -73,7 +74,7 @@ impl<R: Read> Decoder<R> {
 
     /// Recommendation for the size of the output buffer.
     pub fn recommended_output_size() -> usize {
-        unsafe { ll::ZBUFF_recommendedDOutSize() }
+        unsafe { ll::ZSTD_DStreamOutSize() }
     }
 
     /// Return the inner `Read`.
@@ -89,13 +90,26 @@ impl<R: Read> Read for Decoder<R> {
     fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
 
         if self.offset > self.buffer.capacity() {
-            return Ok(0); // End-of-frame reached.
+            // If we've reached the end of the frame before,
+            // don't even try to read more.
+            return Ok(0);
         }
 
-        let mut written = 0;
-        while written != buf.len() {
+        let mut out_buffer = ll::ZSTD_outBuffer {
+            dst: buf.as_mut_ptr(),
+            size: buf.len(),
+            pos: 0,
+        };
+        let mut in_buffer = ll::ZSTD_inBuffer {
+            src: self.buffer.as_ptr(),
+            size: self.buffer.len(),
+            pos: self.offset,
+        };
+        while out_buffer.pos != buf.len() {
 
-            if self.offset == self.buffer.len() {
+            let mut input_exhausted = false;
+
+            if in_buffer.pos == in_buffer.size {
                 // We need moar data!
                 // Make a nice clean buffer
                 let buffer_size = self.buffer.capacity();
@@ -104,41 +118,39 @@ impl<R: Read> Read for Decoder<R> {
                 }
 
                 // And FILL IT!
-                self.offset = 0;
                 let read = try!(self.reader.read(&mut self.buffer));
                 unsafe {
                     self.buffer.set_len(read);
                 }
-                // If we can't read anything, no need to try and decompress it.
-                // Just break the loop.
+                in_buffer.pos = 0;
+                in_buffer.size = read;
+                // So we can't read anything: input is exhausted.
                 if read == 0 {
-                    break;
+                    input_exhausted = true;
                 }
-
             }
 
-            let mut out_size = buf.len() - written;
-            let mut in_size = self.buffer.len() - self.offset;
-
             let res = unsafe {
                 let code =
-                    ll::ZBUFF_decompressContinue(self.context.c,
-                                                 buf[written..].as_mut_ptr(),
-                                                 &mut out_size,
-                                                 self.buffer[self.offset..]
-                                                     .as_ptr(),
-                                                 &mut in_size);
-                try!(ll::parse_code(code))
+                    ll::ZSTD_decompressStream(self.context.s,
+                                              &mut out_buffer as *mut ll::ZSTD_outBuffer,
+                                              &mut in_buffer as *mut ll::ZSTD_inBuffer);
+                try!(parse_code(code))
             };
 
-            written += out_size;
+            if res > 1 && input_exhausted {
+                // zstd keeps asking for more, but we're short on data!
+                return Err(io::Error::new(io::ErrorKind::UnexpectedEof, "incomplete frame"));
+            }
+
             if res == 0 {
-                // End-of-frame marker.
-                self.offset = self.buffer.capacity() + 1;
+                // Remember that we've reached the end of the current frame,
+                // so we don't try to read the next one.
+                in_buffer.pos = self.buffer.capacity() + 1;
                 break;
             }
-            self.offset += in_size;
         }
-        Ok(written)
+        self.offset = in_buffer.pos;
+        Ok(out_buffer.pos)
     }
 }
